# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2022 John Mille<john@compose-x.io>

import pathlib
import socket
from os import path
from typing import Any

from OpenSSL import crypto

from ecs_files_composer.files_mgmt import File
from ecs_files_composer.input import X509CertDef


class X509Certificate(X509CertDef):
    """
    Class to wrap actions around a new X509 certificate
    """

    def __init__(self, **data: Any):
        super().__init__(**data)
        self.key = None
        self.cert = None
        self.key_content = None
        self.cert_content = None
        self.cert_file = None
        self.key_file = None
        self.cert_file_path = None
        self.key_file_path = None

    def init_cert_paths(self):
        self.cert_file_path = path.abspath(f"{self.dir_path}/{self.cert_file_name}")
        self.key_file_path = path.abspath(f"{self.dir_path}/{self.key_file_name}")
        print(f"Creating {self.dir_path} folder")
        dir_path = pathlib.Path(path.abspath(self.dir_path))
        dir_path.mkdir(parents=True, exist_ok=True)

    def generate_key(self):
        self.key = crypto.PKey()
        self.key.generate_key(crypto.TYPE_RSA, 4096)

    def set_common_name(self):
        if self.common_name is None:
            self.common_name = socket.gethostname()

    def generate_cert(self):
        if not self.common_name:
            self.set_common_name()
        self.cert = crypto.X509()
        self.cert.get_subject().C = self.country_name
        self.cert.get_subject().ST = self.state_or_province_name
        self.cert.get_subject().L = self.locality_name
        self.cert.get_subject().O = self.organization_name
        self.cert.get_subject().OU = self.organization_unit_name
        self.cert.get_subject().CN = self.common_name
        self.cert.get_subject().emailAddress = self.email_address
        self.cert.set_serial_number(0)
        self.cert.gmtime_adj_notBefore(0)
        self.cert.gmtime_adj_notAfter(int(self.validity_end_in_seconds))
        self.cert.set_issuer(self.cert.get_subject())
        self.cert.set_pubkey(self.key)
        self.cert.sign(self.key, "sha512")

    def generate_cert_content(self):
        if not self.key:
            self.generate_key()
        if not self.cert:
            self.generate_cert()
        self.cert_content = crypto.dump_certificate(
            crypto.FILETYPE_PEM, self.cert
        ).decode("utf-8")
        self.key_content = crypto.dump_privatekey(crypto.FILETYPE_PEM, self.key).decode(
            "utf-8"
        )

    def set_cert_files(self):
        if not self.cert_content or not self.key_content:
            self.generate_cert_content()
        self.key_file = File().parse_obj(
            {
                "content": self.key_content,
                "path": self.key_file_path,
                "mode": "0600",
                "owner": self.owner,
                "group": self.group,
            }
        )
        self.cert_file = File().parse_obj(
            {
                "content": self.cert_content,
                "path": self.cert_file_path,
                "mode": "0600",
                "owner": self.owner,
                "group": self.group,
            }
        )


def process_x509_certs(job):
    """

    :param ecs_files_composer.input.Model job:
    :return:
    """
    if not hasattr(job.certificates, "x509") or not job.certificates.x509:
        return
    for cert_path, cert_def in job.certificates.x509.items():
        cert_obj = X509Certificate(
            **(cert_def.dict(by_alias=True)),
        )
        cert_obj.dir_path = cert_path
        cert_obj.init_cert_paths()
        cert_obj.set_cert_files()
        job.certificates.x509[cert_path] = cert_obj
        job.files[cert_obj.cert_file.path] = cert_obj.cert_file
        job.files[cert_obj.key_file_path] = cert_obj.key_file



# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2022 John Mille<john@compose-x.io>

"""AWS module."""

import re

import boto3
from boto3.session import Session
from botocore.exceptions import ClientError

from ecs_files_composer import input
from ecs_files_composer.common import LOG
from ecs_files_composer.envsubst import expandvars


def create_session_from_creds(tmp_creds: dict, region: str = None):
    """
    Function to easily convert the AssumeRole reply into a boto3 session
    :param tmp_creds:
    :return:
    :rtype boto3.session.Session
    """
    creds = tmp_creds["Credentials"]
    params = {
        "aws_access_key_id": creds["AccessKeyId"],
        "aws_secret_access_key": creds["SecretAccessKey"],
        "aws_session_token": creds["SessionToken"],
    }
    if region:
        params["region_name"] = region
    return boto3.session.Session(**params)


def set_session_from_iam_object(iam_config_object, source_session: Session = None):
    """
    Function to define the client session based on config input

    :param ecs_files_composer.input.IamOverrideDef iam_config_object:
    :param source_session:
    :return: boto session
    :rtype: boto3.session.Session
    """
    if source_session is None:
        source_session = boto3.Session()
    if not iam_config_object.access_key_id and not iam_config_object.secret_access_key:
        params = {
            "RoleArn": iam_config_object.role_arn,
            "RoleSessionName": f"{iam_config_object.session_name}@AwsResourceHandlerInit",
        }
        if iam_config_object.external_id:
            params["ExternalId"] = iam_config_object.external_id
        tmp_creds = source_session.client("sts").assume_role(**params)
        client_session = create_session_from_creds(
            tmp_creds, region=iam_config_object.region_name
        )
    else:
        client_session = boto3.session.Session(
            aws_access_key_id=iam_config_object.access_key_id,
            aws_secret_access_key=iam_config_object.secret_access_key,
            aws_session_token=iam_config_object.session_token
            if iam_config_object.session_token
            else None,
        )
    return client_session


class AwsResourceHandler:
    """
    Class to handle all AWS related credentials init.
    """

    def __init__(
        self,
        role_arn=None,
        external_id=None,
        region=None,
        iam_config_object=None,
        client_session_override=None,
    ):
        """
        :param str role_arn:
        :param str external_id:
        :param str region:
        :param ecs_files_composer.input.IamOverrideDef iam_config_object:
        """
        self.session = Session()
        self.client_session = Session()
        if client_session_override:
            self.client_session = client_session_override
        elif not client_session_override and (role_arn or iam_config_object):
            if role_arn and not iam_config_object:
                params = {
                    "RoleArn": role_arn,
                    "RoleSessionName": "EcsConfigComposer@AwsResourceHandlerInit",
                }
                if external_id:
                    params["ExternalId"] = external_id
                tmp_creds = self.session.client("sts").assume_role(**params)
                self.client_session = create_session_from_creds(
                    tmp_creds, region=region
                )
            elif (
                iam_config_object
                and hasattr(iam_config_object, "role_arn")
                and iam_config_object.role_arn
            ):
                self.client_session = set_session_from_iam_object(
                    iam_config_object, self.session
                )


class S3Fetcher(AwsResourceHandler):
    """
    Class to handle S3 actions
    """

    bucket_re = re.compile(r"^s3://(?P<bucket>[a-zA-Z\d\-.]+)/(?P<key>[\S]+)$")
    compose_x_re = re.compile(r"^(?P<bucket>[a-zA-Z\d\-.]+)::(?P<key>[\S]+)$")

    def __init__(
        self,
        role_arn=None,
        external_id=None,
        region=None,
        iam_config_object=None,
        client_session_override=None,
    ):
        super().__init__(
            role_arn, external_id, region, iam_config_object, client_session_override
        )

    @property
    def client(self):
        return self.client_session.client("s3")

    def get_content(
        self,
        s3_uri: str = None,
        s3_bucket: str = None,
        s3_key: str = None,
        composex_uri: str = None,
    ):
        """
        Retrieves a file in a temp dir and returns content

        :return: The Stream Body for the file, allowing to do various things
        """

        if s3_uri and self.bucket_re.match(s3_uri):
            s3_bucket = self.bucket_re.match(s3_uri).group("bucket")
            s3_key = self.bucket_re.match(s3_uri).group("key")
        elif composex_uri and self.compose_x_re.match(composex_uri):
            s3_bucket = self.compose_x_re.match(composex_uri).group("bucket")
            s3_key = self.compose_x_re.match(composex_uri).group("key")
        try:
            file_r = self.client.get_object(Bucket=s3_bucket, Key=s3_key)
            file_content = file_r["Body"]
            return file_content
        except self.client.exceptions.NoSuchKey:
            LOG.error(f"Failed to download the file {s3_key} from bucket {s3_bucket}")
            raise


class SsmFetcher(AwsResourceHandler):
    """
    Class to handle SSM actions
    """

    arn_re = re.compile(
        r"^arn:aws(?:-[a-z]+)?:ssm:[\S]+:[\d]{12}:parameter(?P<name>/[\S]+)$"
    )

    def __init__(
        self,
        role_arn=None,
        external_id=None,
        region=None,
        iam_config_object=None,
        client_session_override=None,
    ):
        super().__init__(
            role_arn, external_id, region, iam_config_object, client_session_override
        )
        self.client = self.client_session.client("ssm")

    def get_content(self, parameter_name):
        """
        Import the Content of a given parameter
        If the parameter name is a valid ARN, parses and uses the name from ARN

        :param parameter_name:
        :return:
        """
        if self.arn_re.match(parameter_name):
            parameter_name = self.arn_re.match(parameter_name).group("name")
        try:
            parameter = self.client.get_parameter(
                Name=parameter_name, WithDecryption=True
            )
            return parameter["Parameter"]["Value"]
        except self.client.exceptions:
            raise
        except ClientError:
            raise


class SecretFetcher(AwsResourceHandler):
    """
    Class to handle Secret Manager actions
    """

    def __init__(
        self,
        role_arn=None,
        external_id=None,
        region=None,
        iam_config_object=None,
        client_session_override=None,
    ):
        super().__init__(
            role_arn, external_id, region, iam_config_object, client_session_override
        )
        self.client = self.client_session.client("secretsmanager")

    def get_content(self, secret):
        """
        Import the Content of a given parameter

        :param input.SecretDef secret:
        :return:
        """
        secret_id = expandvars(secret.secret_id)
        params = {"SecretId": secret_id}
        LOG.debug(f"Retrieving secretsmanager://{secret_id}")
        if secret.version_id:
            params["VersionId"] = secret.version_id
        if secret.version_stage:
            params["VersionStage"] = secret.version_stage
        try:
            parameter = self.client.get_secret_value(**params)
            return parameter["SecretString"]
        except self.client.exceptions:
            raise
        except ClientError:
            raise



# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2022 John Mille<john@compose-x.io>

"""Main module."""

import base64
import os
import pathlib
import subprocess
import warnings
from os import path
from tempfile import TemporaryDirectory
from typing import Any

import jinja2.exceptions
import requests
from botocore.response import StreamingBody
from compose_x_common.compose_x_common import keyisset
from jinja2 import Environment, FileSystemLoader

from ecs_files_composer.aws_mgmt import S3Fetcher, SecretFetcher, SsmFetcher
from ecs_files_composer.common import LOG
from ecs_files_composer.envsubst import expandvars
from ecs_files_composer.input import Context, Encoding, FileDef, IgnoreFailureItem
from ecs_files_composer.jinja2_filters import JINJA_FILTERS, JINJA_FUNCTIONS


class File(FileDef):
    """
    Class to wrap common files actions around
    """

    def __init__(self, **data: Any):
        super().__init__(**data)
        self.templates_dir = None

    def handler(self, iam_override=None, session_override=None):
        """
        Main entrypoint for files to relate

        :param ecs_files_composer.input.IamOverrideDef iam_override:
        :param boto3.session.Session session_override:
        """
        if not path.exists(self.dir_path):
            print(f"Creating {self.dir_path} folder")
            dir_path = pathlib.Path(path.abspath(self.dir_path))
            dir_path.mkdir(parents=True, exist_ok=True)
        if self.commands and self.commands.pre:
            warnings.warn("Commands are not yet implemented", Warning)
        if (
            self.context
            and isinstance(self.context, Context)
            and self.context.value == "jinja2"
        ):
            self.templates_dir = TemporaryDirectory()
        if self.source and not self.content:
            retrieved, ignore = self.handle_sources(
                iam_override=iam_override, session_override=session_override
            )
            if not retrieved and ignore:
                LOG.warn(
                    f"Failed to fetch content for {self.path}. Ignoring all post processing."
                )
                return
            elif not retrieved and not ignore:
                raise Exception("Failed to retrieve content from source", self.path)
        self.files_content_processing()
        self.post_processing()

    def files_content_processing(self) -> None:

        if self.content and self.encoding and self.encoding == Encoding["base64"]:
            self.content = base64.b64decode(self.content).decode()
        if self.templates_dir:
            self.write_content(is_template=True)
            self.render_jinja()
        else:
            self.write_content(is_template=False)

    def post_processing(self):
        self.set_unix_settings()
        if self.commands and self.commands.post:
            self.exec_post_commands()

    @property
    def dir_path(self) -> str:
        return path.abspath(path.dirname(self.path))

    def handle_sources(
        self, iam_override=None, session_override=None
    ) -> tuple[bool, bool]:
        """
        Handles files from external sources

        :param ecs_files_composer.input.IamOverrideDef iam_override:
        :param boto3.session.Session session_override:
        """
        ignore_source_download_error = (
            self.ignore_failure
            if self.ignore_failure and isinstance(self.ignore_failure, bool)
            else False
        )
        if self.ignore_failure and isinstance(self.ignore_failure, IgnoreFailureItem):
            ignore_source_download_error = self.ignore_failure.source_download
        retrieved = False
        if self.source.url:
            retrieved = self.handle_url_source()
        elif self.source.ssm:
            retrieved = self.handle_ssm_source(iam_override, session_override)
        elif self.source.s3:
            retrieved = self.handle_s3_source(iam_override, session_override)
        elif self.source.secret:
            retrieved = self.handle_secret_source(iam_override, session_override)
        LOG.debug(
            f"Return from source for {self.path}: {retrieved}-{ignore_source_download_error}"
        )
        return retrieved, ignore_source_download_error

    def handle_ssm_source(self, iam_override=None, session_override=None) -> bool:
        """
        Handles retrieving the content from SSM Parameter

        :param ecs_files_composer.input.IamOverrideDef iam_override:
        :param boto3.session.Session session_override:
        :return:
        """
        parameter_name = expandvars(self.source.ssm.parameter_name)
        LOG.debug(f"Retrieving ssm://{parameter_name}")
        if self.source.ssm.iam_override:
            fetcher = SsmFetcher(iam_config_object=self.source.ssm.iam_override)
        elif iam_override:
            fetcher = SsmFetcher(iam_config_object=iam_override)
        elif session_override:
            fetcher = SsmFetcher(client_session_override=session_override)
        else:
            fetcher = SsmFetcher()
        try:
            self.content = fetcher.get_content(parameter_name=parameter_name)
            return True
        except Exception as error:
            LOG.error("Failed to retrieve file from AWS Systems Manager Parameter")
            LOG.error(error)
            return False

    def handle_s3_source(self, iam_override=None, session_override=None) -> bool:
        """
        Handles retrieving the content from S3

        :param ecs_files_composer.input.IamOverrideDef iam_override:
        :param boto3.session.Session session_override:
        :return: bool, result of the download from S3.
        """
        from ecs_files_composer.input import S3Def1

        if not isinstance(self.source.s3.__root__, S3Def1):
            raise TypeError(
                "S3 source is not of type S3Def1", type(self.source.s3.__root__)
            )

        if self.source.s3.__root__.iam_override:
            fetcher = S3Fetcher(iam_config_object=self.source.s3.__root__.iam_override)
        elif iam_override:
            fetcher = S3Fetcher(iam_config_object=iam_override)
        elif session_override:
            fetcher = S3Fetcher(client_session_override=session_override)
        else:
            fetcher = S3Fetcher()
        try:
            if self.source.s3.__root__.s3_uri:
                self.content = fetcher.get_content(
                    s3_uri=self.source.s3.__root__.s3_uri.__root__,
                )
            elif self.source.s3.__root__.compose_x_uri:
                self.content = fetcher.get_content(
                    composex_uri=self.source.s3.__root__.compose_x_uri.__root__,
                )
            else:
                bucket_name = expandvars(self.source.s3.__root__.bucket_name)
                key = expandvars(self.source.s3.__root__.key)
                self.content = fetcher.get_content(
                    s3_bucket=bucket_name,
                    s3_key=key,
                )
            return True
        except Exception as error:
            LOG.error("Failed to retrieve file from AWS S3")
            LOG.error(error)
            return False

    def handle_secret_source(self, iam_override=None, session_override=None) -> bool:
        """
        Handles retrieving secrets from AWS Secrets Manager

        :param ecs_files_composer.input.IamOverrideDef iam_override:
        :param boto3.session.Session session_override:
        :return:
        """
        if self.source.secret.iam_override:
            fetcher = SecretFetcher(iam_config_object=self.source.secret.iam_override)
        elif iam_override:
            fetcher = SecretFetcher(iam_config_object=iam_override)
        elif session_override:
            fetcher = SecretFetcher(client_session_override=session_override)
        else:
            fetcher = SecretFetcher()
        try:
            self.content = fetcher.get_content(self.source.secret)
            return True
        except Exception as error:
            LOG.error("Failed to retrieve file from AWS Secrets Manager")
            LOG.error(error)
            return False

    def handle_url_source(self) -> bool:
        """
        Fetches the content from a provided URI

        """
        if not self.source.url.username or not self.source.url.password:
            req = requests.get(self.source.url.url)
        else:
            req = requests.get(
                self.source.url.url,
                auth=(self.source.url.username, self.source.url.password),
            )
        try:
            req.raise_for_status()
            self.write_content(as_bytes=True, bytes_content=req.content)
            return True
        except requests.exceptions.HTTPError as error:
            LOG.error("Failed to retrieve file provided URL")
            LOG.error(error)
            return False

    def render_jinja(self):
        """
        Allows to use the temp directory as environment base, the original file as source template, and render
        a final template.
        """
        from os import listdir

        LOG.info(f"Rendering Jinja for {self.path} - {self.templates_dir.name}")
        jinja_env = Environment(
            loader=FileSystemLoader(self.templates_dir.name),
            autoescape=True,
            auto_reload=False,
        )
        jinja_env.filters.update(JINJA_FILTERS)
        jinja_env.globals.update(JINJA_FUNCTIONS)
        try:
            template = jinja_env.get_template(path.basename(self.path))
            self.content = template.render(env=os.environ)
            self.write_content(is_template=False)
        except jinja2.exceptions.TemplateNotFound:
            LOG.error(listdir(self.templates_dir.name))
            raise

    def set_unix_settings(self):
        """
        Applies UNIX settings to given file

        """
        ignore_mode_failure = (
            self.ignore_failure
            if self.ignore_failure and isinstance(self.ignore_failure, bool)
            else False
        )
        ignore_owner_failure = (
            self.ignore_failure
            if self.ignore_failure and isinstance(self.ignore_failure, bool)
            else False
        )
        if self.ignore_failure and isinstance(self.ignore_failure, IgnoreFailureItem):
            ignore_mode_failure = self.ignore_failure.mode
        if self.ignore_failure and isinstance(self.ignore_failure, IgnoreFailureItem):
            ignore_mode_failure = self.ignore_failure.owner

        cmd = ["chmod", self.mode, self.path]
        LOG.debug(f"{self.path} - {cmd}")
        try:
            res = subprocess.run(
                cmd,
                text=True,
                capture_output=True,
                shell=False,
            )
        except subprocess.CalledProcessError as error:
            if ignore_mode_failure:
                LOG.error(error)
            else:
                raise
        cmd = ["chown", f"{self.owner}:{self.group}", self.path]
        LOG.debug(f"{self.path} - {cmd}")
        try:
            res = subprocess.run(
                cmd,
                text=True,
                capture_output=True,
                shell=False,
            )
        except subprocess.CalledProcessError as error:
            if ignore_owner_failure:
                LOG.error(error)
            else:
                raise

    def exec_post_commands(self):

        ignore_post_command_failure = (
            self.ignore_failure
            if self.ignore_failure and isinstance(self.ignore_failure, bool)
            else False
        )
        if self.ignore_failure and isinstance(self.ignore_failure, IgnoreFailureItem):
            ignore_post_command_failure = self.ignore_failure.commands
        commands = self.commands.post.__root__
        for command in commands:
            cmd = command
            if isinstance(command, str):
                cmd = command.split(" ")
            LOG.info(f"{self.path} - {cmd}")
            try:
                res = subprocess.run(
                    cmd,
                    text=True,
                    capture_output=True,
                    shell=False,
                )
            except subprocess.CalledProcessError as error:
                if ignore_post_command_failure:
                    LOG.error(error)
                else:
                    raise

    def write_content(self, is_template=True, as_bytes=False, bytes_content=None):
        """
        Function to write the content retrieved to path.

        :param bool is_template: Whether the content should be considered to be a template.
        :param as_bytes:
        :param bytes_content:
        :return:
        """
        file_path = (
            f"{self.templates_dir.name}/{path.basename(self.path)}"
            if (self.templates_dir and is_template)
            else self.path
        )
        LOG.info(f"Outputting {self.path} to {file_path}")
        if isinstance(self.content, str):
            with open(file_path, "w") as file_fd:
                file_fd.write(self.content)
        elif isinstance(self.content, StreamingBody):
            with open(file_path, "wb") as file_fd:
                file_fd.write(self.content.read())
        elif as_bytes and bytes_content:
            with open(file_path, "wb") as file_fd:
                file_fd.write(bytes_content)



# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2022 John Mille<john@compose-x.io>

"""
Package allowing to expand the Jinja filters to use.
"""
import json
import re
from os import environ

import requests
import yaml
from boto3.session import Session
from flatdict import FlatDict, FlatterDict

from .aws_filters import msk_bootstrap


def env_override(value, key):
    """
    Function to use in new Jinja filter
    :param value:
    :param key:
    :return:
    """
    return environ.get(key, value)


def define_metadata(for_task=False):
    meta_v4 = "ECS_CONTAINER_METADATA_URI_V4"
    meta_v3 = "ECS_CONTAINER_METADATA_URI"

    if environ.get(meta_v3, None):
        meta_url = environ.get(meta_v3)
    elif environ.get(meta_v4, None):
        meta_url = environ.get(meta_v4)
    else:
        raise OSError("No ECS Metadata URL provided. This filter only works on ECS")
    if for_task:
        return requests.get(f"{meta_url}/task")
    else:
        return requests.get(meta_url)


def from_list_to_dict(top_key, new_mapping, to_convert):
    """

    :param str top_key:
    :param dict new_mapping:
    :param list to_convert:
    :return:
    """
    for count, item in enumerate(to_convert):
        list_key = f"{top_key}_{count}"
        if isinstance(item, dict):
            from_dict_to_simple_keys(list_key, new_mapping, item)
        elif isinstance(item, list):
            from_list_to_dict(list_key, new_mapping, item)


def from_dict_to_simple_keys(top_key, new_mapping, to_convert):
    """

    :param str top_key:
    :param dict new_mapping:
    :param dict to_convert:
    :return:
    """
    for key, value in to_convert.items():
        if isinstance(value, str):
            new_mapping[f"{top_key}_{key}"] = value
        elif isinstance(value, dict):
            from_dict_to_simple_keys(f"{top_key}_{key}", new_mapping, value)
        elif isinstance(value, list):
            from_list_to_dict(f"{top_key}_{key}", new_mapping, value)


def from_metadata_to_flat_keys(metadata):
    """
    Function to transform the metadata into simplified structure
    :param dict metadata:
    :return:
    """
    new_metadata = {}
    for key, value in metadata.items():
        if isinstance(value, str):
            new_metadata[key] = value
        elif isinstance(value, dict):
            from_dict_to_simple_keys(key, new_metadata, value)
        elif isinstance(value, list):
            from_list_to_dict(key, new_metadata, value)
    return new_metadata


def get_property(metadata, property_key, separator: str = None):
    if separator is None:
        separator = r"::"
    metadata_mapping = FlatterDict(metadata)
    metadata_mapping.set_delimiter(separator)
    if property_key in metadata_mapping:
        return metadata_mapping[property_key]

    metadata_mapping = from_metadata_to_flat_keys(metadata)
    property_re = re.compile(property_key)
    for key, value in metadata_mapping.items():
        if property_re.findall(key):
            return value
    return None


def ecs_container_metadata(property_key=None, fallback_value=None):
    metadata_raw = define_metadata()
    metadata = metadata_raw.json()
    if property_key:
        value = get_property(metadata, property_key)
        if value is None:
            print(f"No task property found matching {property_key}")
            return fallback_value
        return value
    return metadata


def ecs_task_metadata(property_key=None, fallback_value=None):
    metadata_raw = define_metadata(for_task=True)
    metadata = metadata_raw.json()
    if property_key:
        value = get_property(metadata, property_key)
        if value is None:
            print(f"No task property found matching {property_key}")
            return fallback_value
        return value
    return metadata


def to_yaml(value):
    """
    Filter to render input to YAML formatted content
    :return:
    """
    return yaml.dump(value, Dumper=yaml.Dumper)


def to_json(value, indent=2):
    return json.dumps(value, indent=indent)


def env_var(key, value=None):
    return environ.get(key, value)


def from_ssm(parameter_name: str) -> str:
    """
    Function to retrieve an SSM parameter value

    :param parameter_name: Name of the parameter
    """
    return (
        Session()
        .client("ssm")
        .get_parameter(Name=parameter_name, WithDecryption=True)["Parameter"]["Value"]
    )


def from_ssm_json(parameter_name: str) -> dict:
    """
    Function to retrieve an SSM parameter value

    :param parameter_name: Name of the parameter
    """
    value_str = (
        Session()
        .client("ssm")
        .get_parameter(Name=parameter_name, WithDecryption=True)["Parameter"]["Value"]
    )
    try:
        return json.loads(value_str)
    except json.JSONDecodeError:
        return {}


def hostname(alternative_value: str = None) -> str:
    try:
        import platform

        return str(platform.node())
    except Exception as error:
        print("Error with platform", error)
        try:
            import socket

            return str(socket.gethostname())
        except Exception as error:
            print("Error with socket", error)
            pass
    if alternative_value:
        return alternative_value


JINJA_FUNCTIONS = {
    "ecs_container_metadata": ecs_container_metadata,
    "ecs_task_metadata": ecs_task_metadata,
    "env_var": env_var,
    "from_ssm": from_ssm,
    "from_ssm_json": from_ssm_json,
    "msk_bootstrap": msk_bootstrap,
    "hostname": hostname,
}

JINJA_FILTERS = {"to_yaml": to_yaml, "to_json": to_json, "env_override": env_override}
